-- ------------------------------------------------------------------
-- Primary Variables
-- ------------------------------------------------------------------
Altometer = Altometer or {}
Altometer.context = UI.CreateContext("Context")
Altometer.window = {}
Altometer.window.main = UI.CreateFrame("Frame", "MainWindow", Altometer.context)

Altometer.playerName = Altometer.playerName or nil
Altometer.playerShard = Altometer.playerShard or nil
Altometer.playerFaction = Altometer.playerFaction or nil
Altometer.isInitialReadComplete = false -- This is a workaround for Inspect.Item.Slot not being able to see in bags when the Addon is first loaded
Altometer.tooltip = "nil"
Altometer.version = 0.0301
Altometer.myName = ""
Altometer.debug = true

local abilitiesTable = {}
local allAbil = {}


-- ------------------------------------------------------------------
-- Version Checking
-- ------------------------------------------------------------------
function Altometer.version_checking()
	if not alt_known_version then
		-- A version before 0.3.1 will not have this variable
		alt_playerInfo = nil
		print ("Due to incompatibilities with this version and previous version, your character data has been cleared.")
	end
	alt_known_version = Altometer.version
end

-- ------------------------------------------------------------------
-- Startup SavedVariable checks
-- ------------------------------------------------------------------
if alt_windowSettings == nil then
	alt_windowSettings = {}
	alt_windowSettings.state = 'show'
	alt_windowSettings.posX = 700
	alt_windowSettings.posY = 300
	alt_windowSettings.width = 400
	alt_windowSettings.height = 550
end

if alt_playerInfo == nil then
	alt_playerInfo = {}
end


-- ------------------------------------------------------------------
-- Debug Functions
-- ------------------------------------------------------------------
local function initGlobals()
	alt_playerInfo = {}
	alt_windowSettings = {}
	alt_windowSettings.state = 'show'
	alt_windowSettings.posX = 700
	alt_windowSettings.posY = 300
	alt_windowSettings.width = 400
	alt_windowSettings.height = 550
end
local function sayHello(params)
	-- Greet the world (or just some random user)
	if (params == '') then 
		print("hello world!")
	else 
		print("hello " .. params)
	end
	
end
local function altShowAllPlayer()
	if (alt_playerInfo) then
		for k, v in pairs(alt_playerInfo) do
			print (v.name)
		end
	end
end


-- ------------------------------------------------------------------
-- Player Data Manipulation
-- ------------------------------------------------------------------
local function altSavePlayerData()
	pShard = Inspect.Shard()
	pInfo = Inspect.Unit.Detail("player")
	pInfo.shard = pShard.name
	if (alt_playerInfo) then
	else
		alt_playerInfo = {}
	end
	if (alt_playerInfo[pInfo.shard]) then
	else
		alt_playerInfo[pInfo.shard] = {}
	end
	if (alt_playerInfo[pInfo.shard][pInfo.name]) then
	else
		alt_playerInfo[pInfo.shard][pInfo.name] = {}
	end
	alt_playerInfo[pInfo.shard][pInfo.name]['Info'] = pInfo
	alt_playerInfo[pInfo.shard][pInfo.name]['Achievments'] = getAchievementData()
	alt_playerInfo[pInfo.shard][pInfo.name]['Items'] = getItemData()
end
local function altGetPlayerData()
	if (alt_playerInfo) then
		pInfo = alt_playerInfo[Inspect.Shard().name][Inspect.Unit.Detail("player").name]
		if (pInfo) then
			print ("shard", pInfo.shard)
			print ("manaMax", pInfo.manaMax)
			print ("planar", pInfo.planar)
			print ("planarMax", pInfo.planarMax)
			print ("vitality", pInfo.vitality)

			-- Get full name
			pName = ""
			if (pInfo.titlePrefix) then 
				pName = pInfo.titlePrefix.." "
			end
			pName = pName..pInfo.name
			if (pInfo.titleSuffix) then 
				pName = pName..pInfo.titleSuffix
			end 
			
			pResource = 100
			if (pInfo.manaMax) then 
				pResource = pInfo.manaMax
			end
			
			if (pInfo.guild) then
				pGuild = "<"..pInfo.guild..">"
			end
			pCalling = "Level "..pInfo.level.." "..ucfirst(pInfo.calling)
			print ("Name = ", pName)
			if (pGuild) then 
				print ("Guild = ", pGuild)
			end
			print ("Calling = ", pCalling)
			print ("Health ", pInfo.healthMax)
			print (getResourceName(pInfo.calling), pResource)
		else
			print ("Player information not saved...first use /alt save")
		end
	end
end


-- ------------------------------------------------------------------
-- SimpleMeter Thievery
-- ------------------------------------------------------------------
function alt_CreateFrame(p)
	local frame = UI.CreateFrame(p.type, p.name, p.parent)
	if p.hide then
		frame:SetVisible(false)
	end
	if p.layer then
		frame:SetLayer(p.layer)
	end

	if p.type == "Text" then
		if p.fontSize then
			frame:SetFontSize(p.fontSize)
		else
			frame:SetFontSize(12)
		end
	end

	if p.fontR then
		frame:SetFontColor(p.fontR, p.fontG, p.fontB)
	end
	if p.a then
		frame:SetBackgroundColor(p.r, p.g, p.b, p.a)
	end

	if p.text then
		frame:SetText(p.text)
	end
	if p.wordwrap then
		frame:SetWordwrap(p.wordwrap)
	end

	if p.width then
		frame:SetWidth(p.width)
	elseif p.type == "Text" then
		frame:SetWidth(frame:GetFullWidth())
	end

	if p.height then
		frame:SetHeight(p.height)
	elseif p.type == "Text" then
		frame:SetHeight(frame:GetFullHeight())
	end

	if p.align then
		frame:SetPoint(p.align, p.alignTarget, p.alignTo, p.x, p.y)
	end

	if p.padTarget then
		frame:SetPoint("TOPLEFT", p.padTarget, "TOPLEFT", p.pad * -1, p.pad * -1)
		frame:SetPoint("BOTTOMRIGHT", p.padTarget, "BOTTOMRIGHT", p.pad, p.pad)
	end

	if p.border then
		frame.borderTop = UI.CreateFrame("Frame", p.name .. "_btop", frame)
		frame.borderTop:SetLayer(frame:GetLayer() + 1)
		frame.borderTop:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
		frame.borderTop:SetPoint("TOPLEFT", frame, "TOPLEFT")
		frame.borderTop:SetPoint("BOTTOMRIGHT", frame, "TOPRIGHT", 0, p.border)

		frame.borderBot = UI.CreateFrame("Frame", p.name .. "_bbot", frame)
		frame.borderBot:SetLayer(frame:GetLayer() + 1)
		frame.borderBot:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
		frame.borderBot:SetPoint("TOPLEFT", frame, "BOTTOMLEFT", 0, p.border * -1)
		frame.borderBot:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")

		frame.borderRt = UI.CreateFrame("Frame", p.name .. "_brt", frame)
		frame.borderRt:SetLayer(frame:GetLayer() + 1)
		frame.borderRt:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
		frame.borderRt:SetPoint("TOPLEFT", frame, "TOPLEFT")
		frame.borderRt:SetPoint("BOTTOMRIGHT", frame, "BOTTOMLEFT", p.border, 0)

		frame.borderLt = UI.CreateFrame("Frame", p.name .. "_blt", frame)
		frame.borderLt:SetLayer(frame:GetLayer() + 1)
		frame.borderLt:SetBackgroundColor(p.borderR, p.borderG, p.borderB, p.borderA)
		frame.borderLt:SetPoint("TOPLEFT", frame, "TOPRIGHT", p.border * -1, 0)
		frame.borderLt:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")
	end

	return frame;
end


-- ------------------------------------------------------------------
-- Window Creation
-- ------------------------------------------------------------------
function Altometer.altCreateWindow()
	Altometer.window.main = UI.CreateFrame("SimpleWindow", "MainWindow", Altometer.context)
	Altometer.window.main:SetCloseButtonVisible(true)
	Altometer.window.main:SetPoint("TOPLEFT", UIParent, "TOPLEFT", alt_windowSettings.posX, alt_windowSettings.posY)
	Altometer.window.main:SetTitle("ALTOMETER")
	Altometer.window.main:SetHeight(alt_windowSettings.height)
	Altometer.window.main:SetWidth(alt_windowSettings.width)
	Altometer.window.main:SetWidth(1000)
	
	Altometer.window.main.lastChar = nil
	Altometer.window.main.lastItem = nil
	Altometer.window.main.showCount = 0

	-- Character Frame
	Altometer.window.main.characterFrame = UI.CreateFrame("Frame", "CharFrame", Altometer.window.main)
	Altometer.window.main.characterFrame.title = UI.CreateFrame("Text", "CharTitle", Altometer.window.main.characterFrame)
	Altometer.window.main.characterFrame.title:SetFontColor(1,1,1)
	Altometer.window.main.characterFrame.title:SetPoint("TOPLEFT", Altometer.window.main, "TOPLEFT", 100, 70)
	Altometer.window.main.characterFrame.title:SetText("Character List")
	Altometer.window.main.characterFrame.title:SetFontSize(20)
	Altometer.window.main.characterFrame.title:ResizeToText()
	Altometer.window.main.characterFrame.characterList = UI.CreateFrame("Mask", "CharacterFrame", Altometer.window.main.characterFrame)
	Altometer.window.main.characterFrame.characterList:SetBackgroundColor(0,0,0,.7)
	Altometer.window.main.characterFrame.characterList:SetPoint("TOPLEFT",Altometer.window.main.characterFrame.title,"BOTTOMLEFT",0,0)
	Altometer.window.main.characterFrame.characterList:SetPoint("BOTTOMRIGHT", Altometer.window.main, "BOTTOMRIGHT", -30, -30)
	Altometer.window.main.characterFrame:SetVisible(true)
		
	-- Item Frame
	Altometer.window.main.itemFrame = UI.CreateFrame("Frame", "ItemFrame", Altometer.window.main)
	Altometer.window.main.itemFrame.title = UI.CreateFrame("Text", "ItemTitle", Altometer.window.main.itemFrame)
	Altometer.window.main.itemFrame.title:SetFontColor(1,1,1)
	Altometer.window.main.itemFrame.title:SetPoint("TOPLEFT", Altometer.window.main, "TOPLEFT", 100, 70)
	Altometer.window.main.itemFrame.title:SetText("Items")
	Altometer.window.main.itemFrame.title:SetFontSize(20)
	Altometer.window.main.itemFrame.title:ResizeToText()
---[[
	Altometer.window.main.itemFrame.listScrollView = UI.CreateFrame("SimpleScrollView", "SWT_TestScrollView", Altometer.window.main.itemFrame)
	Altometer.window.main.itemFrame.listScrollView:SetPoint("TOPLEFT", Altometer.window.main.itemFrame.title, "BOTTOMLEFT")
	Altometer.window.main.itemFrame.listScrollView:SetPoint("BOTTOMRIGHT", Altometer.window.main, "BOTTOMRIGHT", -30, -30)
	Altometer.window.main.itemFrame.listScrollView:SetBorder(1, 1, 1, 1, 0)
	Altometer.window.main.itemFrame.itemList = UI.CreateFrame("Frame", "ItemList", Altometer.window.main.itemFrame.listScrollView)
	Altometer.window.main.itemFrame.itemList:SetBackgroundColor(0,0,0,.7)
	Altometer.window.main.itemFrame.listScrollView:SetContent(Altometer.window.main.itemFrame.itemList)
--]]
--[[
	Altometer.window.main.itemFrame.listScrollView = UI.CreateFrame("Frame", "SWT_TestScrollView", Altometer.window.main.itemFrame)
	Altometer.window.main.itemFrame.listScrollView:SetPoint("TOPLEFT", Altometer.window.main.itemFrame.title, "BOTTOMLEFT")
	Altometer.window.main.itemFrame.listScrollView:SetPoint("BOTTOMRIGHT", Altometer.window.main, "BOTTOMRIGHT", -30, -30)
	--Altometer.window.main.itemFrame.listScrollView:SetBorder(1, 1, 1, 1, 0)
	Altometer.window.main.itemFrame.itemList = UI.CreateFrame("Frame", "ItemList", Altometer.window.main.itemFrame.listScrollView)
	Altometer.window.main.itemFrame.itemList:SetBackgroundColor(0,0,0,.7)
	Altometer.window.main.itemFrame.listScrollView.content = Altometer.window.main.itemFrame.itemList
--]]
	Altometer.window.main.itemFrame:SetVisible(false)
	
	Altometer.window.main.itemFrame.scrollFrame = UI.CreateFrame("RiftScrollbar", "scrollFrame", Altometer.window.main)
	Altometer.window.main.itemFrame.scrollFrame:SetPoint("TOPRIGHT", Altometer.window.main.itemFrame, "TOPLEFT", 0, 0)
	Altometer.window.main.itemFrame.scrollFrame:SetPoint("BOTTOMRIGHT", Altometer.window.main.itemFrame, "BOTTOMLEFT", 0, 0)
	
	
	-- Toolbar Icon Frames
	Altometer.window.main.toolbar = UI.CreateFrame("Frame", "ToolbarFrame", Altometer.window.main)
	Altometer.window.main.toolbar:SetPoint("TOPLEFT", Altometer.window.main, "TOPLEFT", 30, 100)
	Altometer.window.main.toolbar:SetBackgroundColor(0,0,0,.7)
	Altometer.window.main.toolbar:SetWidth(58)
	Altometer.window.main.toolbar:SetPoint("BOTTOMLEFT", Altometer.window.main, "BOTTOMLEFT", 30, -30)
	
	Altometer.window.main.toolbar.charIcon = UI.CreateFrame("Texture", "CharIcon", Altometer.window.main.toolbar)
	Altometer.window.main.toolbar.charIcon:SetPoint("TOPLEFT", Altometer.window.main.toolbar, "TOPLEFT", 5, 5)
	Altometer.window.main.toolbar.charIcon:SetTexture("Rift","ability_icons/absolution_01.dds")
	
	Altometer.window.main.toolbar.bagIcon = UI.CreateFrame("Texture", "BagIcon", Altometer.window.main.toolbar)
	Altometer.window.main.toolbar.bagIcon:SetPoint("TOPLEFT", Altometer.window.main.toolbar.charIcon, "BOTTOMLEFT", 0, 10)
	Altometer.window.main.toolbar.bagIcon:SetTexture("Rift","item_icons/bag12.dds")
	
	Altometer.window.main.toolbar.abilityIcon = UI.CreateFrame("Texture", "AbilityIcon", Altometer.window.main.toolbar)
	Altometer.window.main.toolbar.abilityIcon:SetPoint("TOPLEFT", Altometer.window.main.toolbar.bagIcon, "BOTTOMLEFT", 0, 10)
	Altometer.window.main.toolbar.abilityIcon:SetTexture("Rift","ability_icons/outfitter2.dds")
	Altometer.window.main.toolbar.abilityIcon:SetAlpha(.3)
	
	Altometer.window.main.toolbar.goldIcon = UI.CreateFrame("Texture", "GoldIcon", Altometer.window.main.toolbar)
	Altometer.window.main.toolbar.goldIcon:SetPoint("TOPLEFT", Altometer.window.main.toolbar.abilityIcon, "BOTTOMLEFT", 0, 10)
	Altometer.window.main.toolbar.goldIcon:SetTexture("Rift","ability_icons/merchant1.dds")
	Altometer.window.main.toolbar.goldIcon:SetAlpha(.3)
	
	Altometer.window.main.toolbar.heartIcon = UI.CreateFrame("Texture", "HeartIcon", Altometer.window.main.toolbar)
	Altometer.window.main.toolbar.heartIcon:SetPoint("TOPLEFT", Altometer.window.main.toolbar.goldIcon, "BOTTOMLEFT", 0, 10)
	Altometer.window.main.toolbar.heartIcon:SetTexture("Rift","ability_icons/internalizeheat2.dds")
	Altometer.window.main.toolbar.heartIcon:SetAlpha(.3)
	
	-- Create tooltip for the window
	Altometer.window.main.tooltip = alt_CreateFrame({
		type = "Text", name = "sm_tooltip", parent = Altometer.window.main,
		layer = 10,
		r = 0, g = 0, b = 0, a = 1,
		text = pMouseover, fontSize = 13, wordwrap = true,
		hide = true,
		alignTarget = UIParent, alignTo = "TOPLEFT", align = "TOPLEFT", x = 40, y = 120
	})
	Altometer.window.main.tooltipBG = alt_CreateFrame({
		type = "Frame", name = "sm_tooltip_bg", parent = Altometer.window.main,
		layer = 9,
		r = 0, g = 0, b = 0, a = 1,
		border = 2, borderR = 0.7, borderG = 0.7, borderB = 0.4, borderA = 0.9,
		hide = true,
		padTarget = Altometer.window.main.tooltip, pad = 14 
	})
	function Altometer.window.main:fShowTooltip(text)
		Altometer.window.main.showCount = Altometer.window.main.showCount + 1
		self.tooltip:SetText(text)
		self.tooltip:SetWidth(210);
		self.tooltip:SetHeight(self.tooltip:GetFullHeight())

		self.tooltipBG:SetVisible(true)
		self.tooltip:SetVisible(true)
	end
	function Altometer.window.main:fHideTooltip()
		Altometer.window.main.showCount = Altometer.window.main.showCount - 1
		
		if Altometer.window.main.showCount < 1 then
			self.tooltip:SetVisible(false)
			self.tooltipBG:SetVisible(false)
		end
	end
	function Altometer.window.main:fPositionTooltip(posX,posY)
		self.tooltip:SetPoint("TOPLEFT",UIParent,"TOPLEFT",posX+20,posY+20)
	end
	function Altometer.window.main.fShow()
		Altometer.window.main:SetVisible(true)
		alt_windowSettings.state = 'show'
	end
	function Altometer.window.main.fHide()
		Altometer.window.main:SetVisible(false)
		alt_windowSettings.state = 'hide'
	end
	
	-- Generates the list of characters 
	function Altometer.window.main.fCharacterList()
		pList = {}
		if (alt_playerInfo) then
			for kShard, vCharacters in pairs(alt_playerInfo) do
				pList[kShard] = {}
				for sCharacter,tData in pairs(vCharacters) do
					if tData['Info'] then
						pInfo = tData['Info']
						-- Get full name
						pFullName = ""
						if (pInfo.titlePrefix) then 
							pFullName = pInfo.titlePrefix.." "
						end
						pFullName = pFullName..pInfo.name
						if (pInfo.titleSuffix) then 
							pFullName = pFullName..pInfo.titleSuffix
						end 
						pMouseover = pFullName.."\n"
						if pInfo.guild then 
							pMouseover = pMouseover.."<"..pInfo.guild..">\n"
						end
						pMouseover = pMouseover.."Level "..pInfo.level.."\n"..ucfirst(pInfo.calling)
						pName = pInfo.name..", level "..pInfo.level.." "..ucfirst(pInfo.calling)
						table.insert(pList[kShard],{name = pName, mouseover = pMouseover})
					end
				end
			end
		end
		
		return pList
	end
	function Altometer.window.main.fAddLine(pName, pMouseover, fontSize)
		fontSize = fontSize or 14
		charItem = UI.CreateFrame("Text","CharacterItem",Altometer.window.main.characterFrame.characterList)
		charItem:SetFontSize(fontSize)
		charItem:SetText(pName)
		charItem:ResizeToText()
		charItem:SetWidth(Altometer.window.main.characterFrame.characterList:GetWidth())
		charItem:SetWordwrap(true)
		if Altometer.window.main.lastChar then
			charItem:SetPoint("TOPLEFT",Altometer.window.main.lastChar,"BOTTOMLEFT",0,0)
		else
			charItem:SetPoint("TOPLEFT",Altometer.window.main.characterFrame.characterList,"TOPLEFT",0,0)
		end
		Altometer.window.main.lastChar = charItem
		charItem.mouseoverText = pMouseover

		function charItem.Event:MouseIn()
			if self.mouseoverText then
				Altometer.window.main:fShowTooltip(self.mouseoverText)
			end
		end
		function charItem.Event:MouseOut()
			if self.mouseoverText then
				Altometer.window.main:fHideTooltip()
			end
		end
		function charItem.Event:MouseMove()
			if self.mouseoverText then
				local mouse = Inspect.Mouse()
				Altometer.window.main:fPositionTooltip(mouse.x,mouse.y)
			end
		end
	end
	function Altometer.window.main.fUpdateList()
		charList = Altometer.window.main.fCharacterList()
		
		currShard = ""
		for k,v in pairs(charList) do
			Altometer.window.main.fAddLine(k, nil, 18)
			for s,t in pairs(v) do
				Altometer.window.main.fAddLine("\t"..t.name, t.mouseover)
			end
			Altometer.window.main.fAddLine(" ")
		end
	end

	function Altometer.window.main.fItemList()
		return getListOfItems()
	end
	function Altometer.window.main.fAddItemLine(pName, pIcon, pMouseover, itemID, pRarity, fontSize)
		fontSize = 14
		lineItem = UI.CreateFrame("Frame", "FrameItem", Altometer.window.main.itemFrame.itemList)
		if Altometer.window.main.lastItem then
			lineItem:SetPoint("TOPLEFT",Altometer.window.main.lastItem,"BOTTOMLEFT",0,4)
			lineItem:SetPoint("TOPRIGHT",Altometer.window.main.lastItem,"TOPRIGHT",0,40)
		else
			lineItem:SetPoint("TOPLEFT",Altometer.window.main.itemFrame.itemList,"TOPLEFT",0,0)
			lineItem:SetPoint("TOPRIGHT",Altometer.window.main.itemFrame.itemList,"TOPRIGHT",0,0)
		end
		itemMask = UI.CreateFrame("Mask","ItemMask",Altometer.window.main.itemFrame.itemList)
		itemMask:SetPoint("TOPLEFT",lineItem,"TOPLEFT",0,0)
		itemMask:SetPoint("BOTTOMRIGHT",lineItem,"BOTTOMRIGHT",0,0)
		itemMask:SetBackgroundColor(1,1,1,0)
		
		itemIcon = UI.CreateFrame("Texture","ItemTexture",lineItem)
		itemIcon:SetPoint("TOPLEFT",lineItem,"TOPLEFT",1,1)
		itemIcon:SetTexture("Rift",pIcon)
		itemIcon:SetHeight(38)
		itemIcon:SetWidth(38)
		itemIcon.mask = UI.CreateFrame("Mask","ItemMask",itemIcon)
		itemIcon.mask:SetPoint("TOPLEFT",itemIcon,"TOPLEFT",0,0)
		itemIcon.mask:SetPoint("BOTTOMRIGHT",itemIcon,"BOTTOMRIGHT",0,0)
		itemIcon.mask:SetBackgroundColor(0,0,0,0)
		
		itemItem = UI.CreateFrame("Text","ItemItem",lineItem)
		itemItem:SetPoint("TOPLEFT",itemIcon,"TOPRIGHT",4,10)
		itemItem:SetFontSize(fontSize)
		itemItem:SetText(pName)
		itemItem:ResizeToText()
		itemItem:SetWordwrap(false)
		clr = rarityColor(pRarity)
		itemItem:SetFontColor(clr.r,clr.g,clr.b,1)
		
		Altometer.window.main.lastItem = lineItem
		itemItem.mouseoverText = pMouseover
		itemMask.mouseoverText = pMouseover
		
		function itemMask.Event:MouseIn()
			if self.mouseoverText then
				Altometer.window.main:fShowTooltip(self.mouseoverText)
			end
			Altometer.tooltip = itemID
			Command.Tooltip(itemID)
			itemMask:SetBackgroundColor(1,1,1,1)
		end
		function itemMask.Event:MouseOut()
			if self.mouseoverText then
				Altometer.window.main:fHideTooltip()
			end
			if Altometer.tooltip == itemID then
				Command.Tooltip(nil)
			end
			itemMask:SetBackgroundColor(1,1,1,0)
		end
		function itemMask.Event:MouseMove()
			if self.mouseoverText then
				local mouse = Inspect.Mouse()
				Altometer.window.main:fPositionTooltip(mouse.x,mouse.y)
			end
		end
	end
	function Altometer.window.main.fUpdateItemList()
		itemList = Altometer.window.main.fItemList()
		itemCount = 0
		itemCount = table.count(itemList)
		Altometer.window.main.itemFrame.itemList:SetHeight(itemCount * 40)
		for k,v in pairsByKeys(itemList) do
			mouseoverText = getTooltipForItem(v)
			for s,t in pairs(v.Items) do
				pIcon = t.Icon
				pRarity = t.Rarity
				itemID = t.Type
			end
			Altometer.window.main.fAddItemLine(v.Name, pIcon, mouseoverText, itemID, pRarity)
		end
	end

	-- Toolbar Icons
	function Altometer.window.main.toolbar.bagIcon.Event:LeftClick()
		--[[
		if not Altometer.isInitialReadComplete then
			alt_playerInfo[Altometer.playerShard][Altometer.playerName]['Items'] = getItemData()
		end
		Altometer.window.main.fUpdateItemList()
		--]]
		Altometer.window.items:ShowWindow()
		Altometer.window.main:SetVisible(false)
		
	end
	function Altometer.window.main.toolbar.charIcon.Event:LeftClick()
		Altometer.window.main.fChangeDisplay("character")
	end
	--[[
	function Altometer.window.main.toolbar.abilityIcon.Event:LeftClick()
		print ("Skills Clicked")
	end
	function Altometer.window.main.toolbar.goldIcon.Event:LeftClick()
		print ("Currency Clicked")
	end
	function Altometer.window.main.toolbar.heartIcon.Event:LeftClick()
		print ("Achievement Clicked")
	end
	--]]
	
	-- Toolbar Events
	function Altometer.window.main.toolbar.bagIcon.Event:MouseIn()
		Altometer.window.main:fShowTooltip("Inventory Information")
	end
	function Altometer.window.main.toolbar.bagIcon.Event:MouseOut()
		Altometer.window.main:fHideTooltip()
	end
	function Altometer.window.main.toolbar.bagIcon.Event:MouseMove()
		local mouse = Inspect.Mouse()
		Altometer.window.main:fPositionTooltip(mouse.x,mouse.y)
	end
	function Altometer.window.main.toolbar.charIcon.Event:MouseIn()
		Altometer.window.main:fShowTooltip("Character Information")
	end
	function Altometer.window.main.toolbar.charIcon.Event:MouseOut()
		Altometer.window.main:fHideTooltip()
	end
	function Altometer.window.main.toolbar.charIcon.Event:MouseMove()
		local mouse = Inspect.Mouse()
		Altometer.window.main:fPositionTooltip(mouse.x,mouse.y)
	end
	--[[
	function Altometer.window.main.toolbar.abilityIcon.Event:MouseIn()
		Altometer.window.main:fShowTooltip("Tradeskill Information")
	end
	function Altometer.window.main.toolbar.abilityIcon.Event:MouseOut()
		Altometer.window.main:fHideTooltip()
	end
	function Altometer.window.main.toolbar.abilityIcon.Event:MouseMove()
		local mouse = Inspect.Mouse()
		Altometer.window.main:fPositionTooltip(mouse.x,mouse.y)
	end
	--]]
	--[[
	function Altometer.window.main.toolbar.goldIcon.Event:MouseIn()
		Altometer.window.main:fShowTooltip("Currency Information")
	end
	function Altometer.window.main.toolbar.goldIcon.Event:MouseOut()
		Altometer.window.main:fHideTooltip()
	end
	function Altometer.window.main.toolbar.goldIcon.Event:MouseMove()
		local mouse = Inspect.Mouse()
		Altometer.window.main:fPositionTooltip(mouse.x,mouse.y)
	end
	--]]
	--[[
	function Altometer.window.main.toolbar.heartIcon.Event:MouseIn()
		Altometer.window.main:fShowTooltip("Achievement Information")
	end
	function Altometer.window.main.toolbar.heartIcon.Event:MouseOut()
		Altometer.window.main:fHideTooltip()
	end
	function Altometer.window.main.toolbar.heartIcon.Event:MouseMove()
		local mouse = Inspect.Mouse()
		Altometer.window.main:fPositionTooltip(mouse.x,mouse.y)
	end
	--]]
	
	function Altometer.window.main.fChangeDisplay(displayThis)
		Altometer.window.main.characterFrame:SetVisible(displayThis == "character")
		Altometer.window.main.itemFrame:SetVisible(displayThis == "item")
	end
	
	-- Set Altometer.window.main visibility
	Altometer.window.main:SetVisible(alt_windowSettings.state == 'show')
	
end


-- ------------------------------------------------------------------
-- Initialization Function
-- ------------------------------------------------------------------
local function loadSettings()
	Altometer.myName = Inspect.Addon.Current()
	print ("Welcome to " .. Altometer.myName .. "!")
	if alt_windowSettings.state then
		if alt_windowSettings.state == 'hide' then
			print (Altometer.myName .. " is currently hidden.  Use /alt show to display the window")
		end
	else
		alt_windowSettings.state = 'show'
	end
	Altometer.altCreateWindow()
	Altometer.createAuctionStyleItemWindow()
end
local function altPlayerAvailable(params)
	for k,v in pairs(params) do
		if v=="player" then
			Altometer.playerName = Inspect.Unit.Detail("player").name
			Altometer.playerFaction = Inspect.Unit.Detail("player").faction
			Altometer.playerShard = Inspect.Shard().name
			altSavePlayerData()
			Altometer.window.main.fUpdateList()
			Altometer.window.main.fUpdateItemList()

			table.insert (Event.Achievement.Complete, {altAchievementComplete, "Altometer", "AchievementComplete"})
			table.insert (Event.Achievement.Update, {altAchievementUpdate, "Altometer", "AchievementUpdate"})
			table.insert (Event.Item.Slot, {itemSlotUpdate, "Altometer", "ItemSlotUpdated"})
			table.insert (Event.Item.Update, {itemSlotUpdate, "Altometer", "ItemUpdated"})
		end
	end
end


-- ------------------------------------------------------------------
-- Ability Functions
-- ------------------------------------------------------------------
local function iterateAbility(abil)
	for key,val in pairs(abil) do
		
	end
	print (detail.name, detail.icon, detail.currentCooldown, detail.currentCooldownRemaining)
end
local function storeAbilities(abil)
	detail = Inspect.Ability.Detail(abil)
	abilitiesTable = {}
	table.insert(abilitiesTable, detail)
	table.foreach (detail, iterateAbility)
	
	end
local function getAbilities()
	abilities = Inspect.Ability.List()
	--table.foreach (abilities, storeAbilities)
	for key,val in pairs(abilities) do
		abil = Inspect.Ability.Detail(key)
		print (Altometer.myName .. ":",key,abil.name)
	end
end
local function getAllAbilityIds(commandline)
	local x, y = commandline:match("(%d+) (%d+)")
	x, y = tonumber(x), tonumber(y)
	if not x or not y then
		print("Must give at two numbers")
		return
	end
	if x>y then
		print ("First number must be less than second number")
		return
	end
	if x<0 or y<0 then
		print("Numbers must be positive")
		return
	end
	--allAbil = {}
	--decMax = tonumber("ffffffff",16)
	--print (DEC_HEX(decMax))
	for i=x,y do
		number = i
		key = DEC_HEX(i)
		abil = Inspect.Ability.Detail(key)
		if abil ~= nil then
			allAbil[key]=abil.name
			print ("Altometer:"..key..",'"..abil.name.."'")
		end
	end
	print ("Scanned from "..x.." to "..y)
end


-- ------------------------------------------------------------------
-- Slash handler
-- ------------------------------------------------------------------
local function altSlashCommand(params)
	tParams = explode(" ",params)
	if tParams[1] == "abil" then
		getAbilities()
	elseif tParams[1] == "show" then
		Altometer.window.main.fShow()
	elseif tParams[1] == "hide" then
		Altometer.window.main.fHide()
	elseif tParams[1] == "gg" then
		sayHello(tParams[2])
	elseif tParams[1] == "mage" then
		checkMageArmor()
	elseif tParams[1] == "track" then
		checkTracking()
	elseif tParams[1] == "player" then
		altGetPlayerData()
	elseif tParams[1] == "save" then
		altSavePlayerData()
	elseif tParams[1] == "init" then
		initGlobals()
	elseif tParams[1] == "all" then
		altShowAllPlayer()
	elseif tParams[1] == "ach" then
		achList()
	elseif tParams[1] == "items" then
		getItemData()
	elseif tParams[1] == "cats" then
		Altometer.createAuctionStyleItemWindow()
	else
		print ("Parameters:")
		print ("\tshow -- show the Altometer window")
		print ("\thide -- hide the Altometer window")
		print ("\tplayer -- Displays player data in SavedVariables")
		print ("\tsave -- Saves player data to SavedVariables")
		print ("\tinit -- Clears player data from SavedVariables")
		print ("\tall -- Lists all data in SavedVariables")
		print ("\tach -- Lists achievements")
		
	end
end


-- ------------------------------------------------------------------
-- Table Insertions
-- ------------------------------------------------------------------
table.insert (Command.Slash.Register("alt"), {altSlashCommand, "Altometer", "altSlashCommand"})
table.insert (Event.Addon.Startup.End, {loadSettings, "Altometer", "loadSettings"})
table.insert (Event.Unit.Available, {altPlayerAvailable, "Altometer", "PlayerAvailable"})
table.insert (Event.Addon.SavedVariables.Load.End, {Altometer.version_checking, "Altometer", "VersionChecking"})




-- SWT_Window = UI.CreateFrame("SimpleWindow", "SWT_Window", Altometer.context)
-- SWT_Window:SetCloseButtonVisible(true)
-- SWT_Window:SetWidth(650)
-- SWT_Window:SetTitle("List Test")
-- SWT_Window:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 100, 100)
-- SWT_Window.listScrollView = UI.CreateFrame("SimpleScrollView", "SWT_TestScrollView", SWT_Window:GetContent())
-- SWT_Window.listScrollView:SetPoint("TOPLEFT", SWT_Window:GetContent(), "TOPLEFT")
-- SWT_Window.listScrollView:SetWidth(150)
-- SWT_Window.listScrollView:SetHeight(300)
-- SWT_Window.listScrollView:SetBorder(1, 1, 1, 1, 1)
-- SWT_Window.list = UI.CreateFrame("SimpleList", "SWT_TestList", SWT_Window.listScrollView)
-- SWT_Window.list.Event.ItemSelect = function(view, item) print("ItemSelect("..item..")") end
-- local items = {}
-- for i=1,100 do
  -- table.insert(items, "Item "..i)
-- end
-- SWT_Window.list:SetItems(items)
-- SWT_Window.listScrollView:SetContent(SWT_Window.list)

